@@ -3,7 +3,7 @@
 #' @param from_nominal_price (numeric) the nominal prices to be converted to a real price
 #' @param from_date (character, date-like) the 'date' contemporaneous to \code{from_nominal_price}. The acceptable forms are 'YYYY', 'YYYY-YY' (financial year), 'YYYY-MM-DD', and 'YYYY-Q[1-4]' (quarters). Note a vector cannot contain a mixture of date forms.
 #' @param to_date (character, date-like) the date at which the real price is valued (where the nominal price equals the real price). Same forms as for \code{from_date}
-#' @param ... other arguments passed to \code{cpi_inflator_quarters}
+#' @param ... other arguments passed to \code{\link{cpi_inflator_quarters}}
 #' 
 #' @return A vector of real prices in \code{to_date} dollars.
 #' @export
@@ -18,7 +18,7 @@ cpi_inflator_general_date <- function(from_nominal_price = 1,
     message("CPI: Using Q4 for each year")
   } else {
     
-    if (all(is.fy(from_date))){
+    if (all_fy(from_date)) {
       from_date <- paste0(fy2yr(from_date), "-Q1")
     } else {
       
